* 0.1.1 - jeremy freese 5/2/05 - small bug fix
* 0.1.0 - jeremy freese

program define case2choice
    
    version 9
    
    syntax [varlist(default=none)] , [y(varname numeric) YRank(name) id(varname) Generate(name) Replace Alt(namelist) noNames]


    * either y() or yrank() must be specified
    if "`y'" == "" & "`yrank'" == "" {
        di as err "option y() or yrank() must be specified"
        exit 198
    }

    * figure out if rankdata is being specified
    local rankdata "yes"
    if "`y'" != "" {
        local rankdata "no"
    }

    if "`rankdata'" == "no" & "`generate'" == "" & "`replace'" == "" {
        di as txt "(note: " as res "choice " as txt "used for outcome since no variable specified by gen())"
        confirm new variable choice
        local generate "choice"
    }
    local choice "`generate'"

    if "`id'" == "" {
        di as txt "(note: variable " as res "_id" as txt " used since no id() variable specified)"
        confirm new variable _id
        gen _id = _n
        local id "_id"
    }


    if "`rankdata'" == "no" {           
        _pecats `y'

        local catval = r(catvals)
        local catnms = r(catnms)
        local numcats = r(numcats)

        forvalues i = 1(1)`numcats' {
            local y`i' : word `i' of `catval'
            local ynm`i' : word `i' of `catnms'
            confirm integer number `y`i''
            gen _ytemp`y`i'' = 1
        }

        local ytemp "_ytemp"
    }

    if "`rankdata'" == "yes" {           
		capture label drop _optnum
		foreach rvar of varlist `yrank'* {
			local rvarnum = subinstr("`rvar'", "`yrank'", "", 1)
			local rvarnumchk = real("`rvarnum'")
			if "`rvarnum'" == "`rvarnumchk'" {
				local rvarlabel : variable label `rvar'
				label define _optnum `rvarnum' "`rvarlabel'", modify
			}
		}
	} 
	
    qui reshape long `ytemp' `yrank' `alt', i(`id') j(_optnum) 
    
    capture drop _ytemp


    if "`rankdata'" == "no" {           

        tempvar ychosen
        gen `ychosen' = `y'

        if "`replace'" == "replace" {
            drop `y'
            local choice "`y'"
        }

        gen `choice' = (_optnum == `ychosen') if _optnum < .

    }

    if "`rankdata'" == "yes" {           
		label define _optnum, modify
		label values _optnum _optnum
        _pecats _optnum

        local catval = r(catvals)
        local catnms = r(catnms)
        local numcats = r(numcats)

        forvalues i = 1(1)`numcats' {
            local y`i' : word `i' of `catval'
            local ynm`i' : word `i' of `catnms'
            confirm integer number `y`i''
        }

		* if generate option specified, rename stub to that
		if "`generate'" != "" {
			rename `yrank' `generate'			
			local yrank "`generate'"
		}

    }


    forvalues i = 1(1)`numcats' {   
		local name = "y`y`i''"
			if "`names'" != "nonames" & "`ynm`i''" != "`y`i''" {
				local name "`ynm`i''"
			}
        qui gen ytemp`y`i'' = (_optnum == `y`i'') if _optnum < .
        label variable ytemp`y`i'' "`ynm`i''"
        if "`varlist'" != "" {
            foreach var of varlist `varlist' {
                qui gen `name'X`var' = `var' * ytemp`y`i''
            }
        }
        local ylist "`ylist' `name'*"
        clonevar `name' = ytemp`y`i''
        drop ytemp`y`i''        
    }


    * output

    if "`rankdata'" == "no" {       
        di _n as txt "choice indicated by:" as res " `choice'"
    }
    if "`rankdata'" == "yes" {
        di _n as txt "ranks indicated by:" as res " `yrank'"
    }
    di as txt "case identifier:" as res " `id'"
    di as txt "case-specific interactions:" as res "`ylist'"
    if "`alt'" != "" {
        di as txt "alternative-specific variables:" as res " `alt'"
    }

end
